'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /* global module */

var _seqr = require('seqr');

var _seqr2 = _interopRequireDefault(_seqr);

var _emitus = require('emitus');

var _emitus2 = _interopRequireDefault(_emitus);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = function () {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var defaults = setDefaults(options);

  var emitter = (0, _emitus2.default)({
    show: show,
    hide: hide,
    setText: setText,
    getElement: getElement,
    setPrefixClass: setPrefixClass,
    setDuration: setDuration,
    setZIndex: setZIndex
  });

  var seqr = (0, _seqr2.default)();
  var cls = getClasses();

  var wrapper = el();
  wrapper.className = cls.prefix + ' ' + cls.hide;
  wrapper.innerHTML = '\n    <span>\n      <span>\n        <span class="' + cls.text + '">' + defaults.text + '</span>\n        <span class="' + cls.progressbar + '"></span>\n      </span>\n    </span>\n  ';

  var spanText = find('.' + cls.text);
  var spanProgressbar = find('.' + cls.progressbar);

  defaults.container.appendChild(wrapper);

  if (defaults.auto) {
    show(defaults.text);
  }

  setZIndex(defaults.zIndex);

  return emitter;

  function show(str) {
    /* istanbul ignore next */
    seqr.then(function (done) {
      setText(str);

      wrapper.classList.remove(cls.hide);

      setTimeout(function () {
        spanText.classList.add(cls.in);
        wrapper.classList.add(cls.in);
      }, 10);

      setTimeout(function () {
        emitter.emit('shown');
        done();
      }, defaults.duration);
    });

    return emitter;
  }

  function hide(fn) {
    /* istanbul ignore next */
    seqr.then(function (done) {
      spanText.classList.remove(cls.in);
      wrapper.classList.remove(cls.in);

      setTimeout(function () {
        wrapper.classList.add(cls.hide);

        if (fn) fn(done);else done();

        emitter.emit('hidden');
      }, defaults.duration);
    });

    return emitter;
  }

  function setText(str) {
    /* istanbul ignore next */
    if (!str && defaults.text) {
      str = defaults.text;
    }

    defaults.text = str;
    spanText.innerHTML = str;

    return emitter;
  }

  function getElement() {
    return wrapper;
  }

  function setDuration(duration) {
    defaults.duration = duration;
  }

  function setZIndex(zindex) {
    defaults.zIndex = zindex;
    wrapper.style.zIndex = zindex;
  }

  function setPrefixClass(prefix) {
    defaults.prefixClass = prefix;
    updateClasses();
  }

  function setDefaults() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return _extends({
      container: document.body,
      duration: 750,
      zIndex: 100,
      auto: false,
      text: 'Loading...',
      prefixClass: 'prelodr'
    }, options);
  }

  function updateClasses() {
    var from = cls;

    cls = getClasses();
    replaceClass(wrapper, from.prefix, cls.prefix);
    replaceClass(spanText, from.text, cls.text);
    replaceClass(spanProgressbar, from.progressbar, cls.progressbar);
  }

  function getClasses() {
    return {
      prefix: defaults.prefixClass,
      in: defaults.prefixClass + '-in',
      hide: defaults.prefixClass + '-hide',
      text: defaults.prefixClass + '-text',
      progressbar: defaults.prefixClass + '-progressbar'
    };
  }

  function el() {
    var tag = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'span';

    return document.createElement(tag);
  }

  function find(q) {
    return wrapper.querySelector(q);
  }

  function replaceClass(elem, from, to) {
    elem.classList.remove(from);
    elem.classList.add(to);
  }
};